/*Author: Michael Fischer, 5 BDH, HTL-Donaustadt, Schuljahr 2004/2005

Summary:
	- Reads the APEv2-Tag from a file
	- Loads the settings from Musepaxx.cfg
	- Substidution of the placeholders with the tags
	- Piping of the decoder and encoder (decoder [settings]|encoder [settings])

Copyright (C) 2005 Michael Fischer

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/

#include "stdafx.h"

#define VERSION "0.1"

#define ARTIST 0
#define YEAR 1
#define ALBUM 2
#define TRACK 3
#define TITLE 4
#define GENRE 5
#define COMMENT 6

#define DLOC 0
#define DSET 1
#define ELOC 2
#define ESET 3

int gettags(char* path,char* tag[])
{
	char *itemkey,*itembuffer;
	char apetag[9]={0}, flags[5]={0}, type[4]={0};
	FILE *fMpc;
    long length;
	int i,j,itemlength;
	int apever,tagsize,items;

	if(!(fMpc = fopen(path, "rb")))		//open .mpc-file
	{
		printf("File not present\n");
		return 0;
	}

	/*fread(type,3,1,fMpc);		//File must be a MP+, disabled for
	if(strcmp("MP+",type)!=0)	//compatibility with other formats
	{
		printf("File not a Musepack-file\n");
		return 0;
	}*/

	fseek(fMpc,0,SEEK_END);				//Calculation of the filesize
	length = ftell(fMpc);
	fseek(fMpc,length-32,SEEK_SET);		//length in bytes

	fread(apetag,sizeof(char)*8,1,fMpc);//8 byte long "APETAGEX" must be present
	if(strcmp("APETAGEX",apetag)!=0)
	{
		printf("APE-Tag not present\n");
		return 0;
	}

	fread(&apever,sizeof(int),1,fMpc);	//APETag Version
	if(apever!=2000)
	{
		printf("APE-Version not correct\n");
		return 0;
	}

	fread(&tagsize,sizeof(int),1,fMpc);	//size of the tag without header
	fread(&items,sizeof(int),1,fMpc);	//number of items
	fread(flags,sizeof(char)*4,1,fMpc);	//flags set

	fseek(fMpc,length - tagsize,SEEK_SET);

	for(i=1;i<=items;i++)
	{
		fread(&itemlength,sizeof(int),1,fMpc);
		itembuffer = new char[itemlength+1];	//setting up the itembuffer

		fseek(fMpc,ftell(fMpc)+4,SEEK_SET);		//flags are not read
		
		itemkey = new char[50];	//reading of the itemkey
		j=0;
		do
		{
			fread(itemkey+j,sizeof(char),1,fMpc);
			j++;
		}while(itemkey[j-1]!=0);

		fread(itembuffer,itemlength,1,fMpc);	//reading of the item value
		itembuffer[itemlength]=0;

		if(tag[TRACK]==NULL)	//saving of the tags into the RAM
		{
			if(strcmp("Track",itemkey)==0)
				tag[TRACK]=itembuffer;
		}
		if(tag[YEAR]==NULL)
		{
			if(strcmp("Year",itemkey)==0)
				tag[YEAR]=itembuffer;
		}
		if(tag[GENRE]==NULL)
		{
			if(strcmp("Genre",itemkey)==0)
					tag[GENRE]=itembuffer;
		}
		if(tag[ARTIST]==NULL)
		{
			if(strcmp("Artist",itemkey)==0)
				tag[ARTIST]=itembuffer;
		}
		if(tag[ALBUM]==NULL)
		{
			if(strcmp("Album",itemkey)==0)
				tag[ALBUM]=itembuffer;
		}
		if(tag[TITLE]==NULL)
		{
			if(strcmp("Title",itemkey)==0)
				tag[TITLE]=itembuffer;
		}
		if(tag[COMMENT]==NULL)
		{
			if(strcmp("Comment",itemkey)==0)
				tag[COMMENT]=itembuffer;
		}				

		//printf("\nItem: %d\n"	//print the info
		//	"Itemkey: %s\n"
		//	"Itemvalue: %s\n",i,itemkey,itembuffer);

		delete itemkey;	//itembuffer doesn't have to be deleted => used by a tag
	}

	//Just for Information
	//printf("\nFiletype: %s\n"
	//	"Filesize: %d\n"
	//	"Tag: %s\n"
	//	"Tagversion: %d\n"
	//	"Tagsize: %d\n"
	//	"Number of Items: %d\n"
	//	"Flags: %s\n",type,length,apetag,apever,tagsize,items,flags);  //Just to check

	printf("\nArist: %s\n"
		"Year: %s\n"
		"Album: %s\n"
		"Track: %s\n"
		"Title: %s\n"
		"Genre: %s\n"
		"Comment: %s\n\n",tag[ARTIST],tag[YEAR],tag[ALBUM],tag[TRACK],tag[TITLE],tag[GENRE],tag[COMMENT]);

	fclose(fMpc);
	return 1;
}


int execute(char* settings[],char* tag[], char* &input, char* &output)
{
	char buffer[2000];
	int i,j,k;

	//strcpy(buffer,"\"");
	strcpy(buffer,settings[DLOC]);
	strcat(buffer," ");
	
	i = strlen(buffer);		//check for %d in decodersettings
	j = 0;
	do
	{
		if(settings[DSET][j]=='%')
		{
			if(settings[DSET][j+1]=='s')
			{
				k=0;
				buffer[i]='\"';
				i++;
				do
				{
                    buffer[i]=input[k];
					k++;
					i++;
				}while(input[k-1]!=0);
				buffer[i-1]='\"';
				buffer[i]=0;
				i=i-1;
				j++;
			}
			else
			{
				buffer[i]=settings[DSET][j];
			}
		}
		else
		{
			buffer[i]=settings[DSET][j];
		}
		i++;
		j++;
	}while(settings[DSET][j-1]!=0);

	strcat(buffer,"|");
	strcat(buffer,settings[ELOC]);
	strcat(buffer," ");

	i = strlen(buffer);		//copying of the tags into the commandline
	j = 0;
    do
	{
		if(settings[ESET][j]=='%')
		{
			if(settings[ESET][j+1]=='a')
			{
				k=0;
				do
				{
                    buffer[i]=tag[ARTIST][k];
					k++;
					i++;
				}while(tag[ARTIST][k-1]!=0);
				i=i-2;
				j++;
			}
			else
			if(settings[ESET][j+1]=='t')
			{
				k=0;
				do
				{
                    buffer[i]=tag[TITLE][k];
					k++;
					i++;
				}while(tag[TITLE][k-1]!=0);
				i=i-2;
				j++;
			}
			else
			if(settings[ESET][j+1]=='g')
			{
				k=0;
				do
				{
                    buffer[i]=tag[ALBUM][k];
					k++;
					i++;
				}while(tag[ALBUM][k-1]!=0);
				i=i-2;
				j++;
			}
			else
			if(settings[ESET][j+1]=='y')
			{
				k=0;
				do
				{
                    buffer[i]=tag[YEAR][k];
					k++;
					i++;
				}while(tag[YEAR][k-1]!=0);
				i=i-2;
				j++;
			}
			else
			if(settings[ESET][j+1]=='n')
			{
				k=0;
				do
				{
                    buffer[i]=tag[TRACK][k];
					k++;
					i++;
				}while(tag[TRACK][k-1]!=0);
				i=i-2;
				j++;
			}
			else
			if(settings[ESET][j+1]=='m')
			{
				k=0;
				do
				{
                    buffer[i]=tag[GENRE][k];
					k++;
					i++;
				}while(tag[GENRE][k-1]!=0);
				i=i-2;
				j++;
			}
			else
			if(settings[ESET][j+1]=='c')
			{
				k=0;
				do
				{
                    buffer[i]=tag[COMMENT][k];
					k++;
					i++;
				}while(tag[COMMENT][k-1]!=0);
				i=i-2;
				j++;
			}
			else
			if(settings[ESET][j+1]=='d')
			{
				k=0;
				buffer[i]='\"';
				i++;
				do
				{
                    buffer[i]=output[k];
					k++;
					i++;
				}while(output[k-1]!=0);
				buffer[i-1]='\"';
				buffer[i]=0;
				i=i-1;
				j++;
			}
			else
			{
				buffer[i]=settings[ESET][j];
			}
		}
		else
		{
			buffer[i]=settings[ESET][j];
		}
		i++;
		j++;
	}while(settings[ESET][j-1]!=0);

	//printf("%s",buffer);
	system(buffer);

	return 1;
}

int loadsettings(char* settings[])
{
	FILE* fMpconf;
	char buffer[500];
	int length;

	if(!(fMpconf=fopen("Musepaxx.cfg","rt")))
	{
		printf("File not present\n");
		return 0;
	}
	
	while(!feof(fMpconf))
	{
		fscanf(fMpconf,"%s",buffer);
		if(buffer[0]=='d')
		{
			if(strcmp(buffer,"decoder_location")==0)
			{
				fseek(fMpconf,ftell(fMpconf)+3,SEEK_SET);
				fgets(buffer,500,fMpconf);
				length=strlen(buffer);
				if(buffer[length-1]==10)
                    buffer[length-1]=0;
				settings[DLOC]=new char[length];
				strcpy(settings[DLOC],buffer);
			}
			else
			{
                if(strcmp(buffer,"decoder_settings")==0)
				{
					fseek(fMpconf,ftell(fMpconf)+3,SEEK_SET);
					fgets(buffer,500,fMpconf);
					length=strlen(buffer);
					if(buffer[length-1]==10)
						buffer[length-1]=0;
					settings[DSET]=new char[length];
					strcpy(settings[DSET],buffer);
				}
			}
		}
		else
		{
			if(buffer[0]=='e')
			{
				if(strcmp(buffer,"encoder_location")==0)
				{
					fseek(fMpconf,ftell(fMpconf)+3,SEEK_SET);
					fgets(buffer,500,fMpconf);
					length=strlen(buffer);
					if(buffer[length-1]==10)
						buffer[length-1]=0;
					settings[ELOC]=new char[length];
					strcpy(settings[ELOC],buffer);
				}
				else
				{
					if(strcmp(buffer,"encoder_settings")==0)
					{
						fseek(fMpconf,ftell(fMpconf)+3,SEEK_SET);
						fgets(buffer,500,fMpconf);
						length=strlen(buffer);
						if(buffer[length-1]==10)
							buffer[length-1]=0;
						settings[ESET]=new char[length];
						strcpy(settings[ESET],buffer);
					}
				}
			}
		}
	}

	if(settings[DLOC]==0||settings[DSET]==0||settings[ELOC]==0||settings[ESET]==0)
	{
		printf("Could not load all settings");
        fclose(fMpconf);
		return 0;
	}

	fclose(fMpconf);
	return 1;
}

int _tmain(int argc, _TCHAR* argv[])
{
	char *tag[7]={0};
	char *settings[4]={0};

	printf("Musepaxx %s\n"
			"Copyright (C) 2005 Michael Fischer\n", VERSION);

	//if nothing was passed through CL, alternative input
	if(argv[1]==NULL||argc<2)
	{
		char inputbuf[50];
		printf("Please enter the input-file: ");
		gets(inputbuf);		//gets is needed because of blanks " "?!
		argv[1]=inputbuf;	//scanf doens't work with " " :(
	}
	if(argv[2]==NULL||argc<3)
	{
		char inputbuf2[50];
		printf("Please enter the output-file: ");
		gets(inputbuf2);	//gets is needed because of blanks " "?!
		argv[2]=inputbuf2;	//scanf doens't work with " " :(
	}
	
	if(!gettags(argv[1],tag))	//reading of the tag-information
	{
		printf("Could not read tag-information");
		getch();
		return 1;
	}
	
	if(!loadsettings(settings))	//loading of Musepaxx.cfg
	{
		printf("Could not open Musepaxx.cfg");
		getch();
		return 1;
	}

	execute(settings,tag, argv[1], argv[2]);

	//getch();

	return 0;
}